@@ -2,7 +2,7 @@
 /**
  *  @brief Mesh is defined as a collection of n-D vertices and simplices
  *
- *  Copyright 2005-2023 Airbus-EDF-IMACS-ONERA-Phimeca
+ *  Copyright 2005-2024 Airbus-EDF-IMACS-ONERA-Phimeca
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -330,7 +330,7 @@ Point Mesh::computeSimplicesVolume() const
     for (UnsignedInteger index = 0; index < nrSimplices; ++index)
     {
       buildSimplexMatrix(index, matrix);
-      result[index] = exp(matrix.computeLogAbsoluteDeterminant(sign, false) - logGamma);
+      result[index] = exp(matrix.computeLogAbsoluteDeterminantInPlace(sign) - logGamma);
     }
   }
   return result;
@@ -464,7 +464,7 @@ void Mesh::fixOrientation(const UnsignedInteger & index,
   }
   buildSimplexMatrix(index, matrix);
   Scalar sign = 0.0;
-  (void) matrix.computeLogAbsoluteDeterminant(sign, false);
+  (void) matrix.computeLogAbsoluteDeterminantInPlace(sign);
   // In odd dimension the positive orientation is for a negative determinant of
   // the simplex matrix
   if ((sign > 0.0) == (dimension_ % 2 == 1)) return;
