@@ -2,7 +2,7 @@
 /**
  *  @brief SORMResult implements the results obtained from the Second Order Reliability Method
  *
- *  Copyright 2005-2023 Airbus-EDF-IMACS-ONERA-Phimeca
+ *  Copyright 2005-2024 Airbus-EDF-IMACS-ONERA-Phimeca
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -342,9 +342,10 @@ String SORMResult::__repr__() const
       << " eventProbabilityBreitung=" << eventProbabilityBreitung_
       << " eventProbabilityHohenbichler=" << eventProbabilityHohenbichler_
       << " eventProbabilityTvedt=" << eventProbabilityTvedt_
-      << " generalisedReliabilityIndexBreitung=" << generalisedReliabilityIndexBreitung_
-      << " generalisedReliabilityIndexHohenbichler=" << generalisedReliabilityIndexHohenbichler_
-      << " generalisedReliabilityIndexTvedt=" << generalisedReliabilityIndexTvedt_
+      // TODO JM: remove the check after the use of infs has been thoroughly tested
+      << " generalisedReliabilityIndexBreitung=" << (generalisedReliabilityIndexBreitung_ < SpecFunc::MaxScalar ? generalisedReliabilityIndexBreitung_ : generalisedReliabilityIndexBreitung_ * 2.0)
+      << " generalisedReliabilityIndexHohenbichler=" << (generalisedReliabilityIndexHohenbichler_ < SpecFunc::MaxScalar ? generalisedReliabilityIndexHohenbichler_ : generalisedReliabilityIndexHohenbichler_ * 2.0)
+      << " generalisedReliabilityIndexTvedt=" << (generalisedReliabilityIndexTvedt_ < SpecFunc::MaxScalar ? generalisedReliabilityIndexTvedt_ : generalisedReliabilityIndexTvedt_ * 2.0)
       << " gradientLimitStateFunction_=" << gradientLimitStateFunction_
       << " hessianLimitStateFunction_=" << hessianLimitStateFunction_;
   return oss;
