@@ -2,7 +2,7 @@
 /**
  *  @brief Subset simulation method
  *
- *  Copyright 2005-2023 Airbus-EDF-IMACS-ONERA-Phimeca
+ *  Copyright 2005-2024 Airbus-EDF-IMACS-ONERA-Phimeca
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -26,7 +26,7 @@
 #include "openturns/ChiSquare.hxx"
 #include "openturns/Normal.hxx"
 #include "openturns/SpecFunc.hxx"
-#include "openturns/ComposedDistribution.hxx"
+#include "openturns/JointDistribution.hxx"
 #include "openturns/Uniform.hxx"
 #include "openturns/RandomGenerator.hxx"
 #include "openturns/MonteCarloExperiment.hxx"
@@ -54,7 +54,6 @@ SubsetSampling::SubsetSampling(const RandomVector & event,
   , proposalRange_(proposalRange)
   , conditionalProbability_(conditionalProbability)
   , minimumProbability_(std::sqrt(SpecFunc::MinScalar))
-  , betaMin_(ResourceMap::GetAsScalar("SubsetSampling-DefaultBetaMin"))
 {
   if (!event.isEvent() || !event.isComposite()) throw InvalidArgumentException(HERE) << "SubsetSampling requires a composite event";
   setMaximumOuterSampling(ResourceMap::GetAsUnsignedInteger("SubsetSampling-DefaultMaximumOuterSampling"));// override simulation default outersampling
@@ -114,41 +113,26 @@ void SubsetSampling::run()
   // blockSize may have changed
   initialExperiment_.setSize(blockSize);
 
+  t0_ = std::chrono::steady_clock::now();
+
   // Step 1: sampling
   for (UnsignedInteger i = 0; i < maximumOuterSampling; ++ i)
   {
-    Sample inputSample;
-    if (!iSubset_)
-    {
-      inputSample = initialExperiment_.generate();
-    }
-    else
-    {
-      // @deprecated: conditional sampling
-      TruncatedDistribution truncatedChiSquare(Chi(dimension_), betaMin_, TruncatedDistribution::LOWER);
-      Normal normal(dimension_);
-      inputSample = Sample(0, dimension_);
-      for (UnsignedInteger j = 0; j < blockSize; ++ j)
-      {
-        Point direction(normal.getRealization());
-        Scalar norm = direction.norm();
-        Scalar radius = truncatedChiSquare.getRealization()[0];
-        if (norm != 0.0)
-        {
-          radius *= 1.0 / norm;
-        }
-        inputSample.add(direction * radius);
-      }
-    }
-    Sample blockSample(standardEvent_.getFunction()(inputSample));
+    const Sample inputSample(initialExperiment_.generate());
+    const Sample blockSample(standardEvent_.getFunction()(inputSample));
     for (UnsignedInteger j = 0 ; j < blockSize; ++ j)
     {
       currentPointSample_[i * blockSize + j] = inputSample[j];
       currentLevelSample_[i * blockSize + j] = blockSample[j];
     }
 
+    std::chrono::steady_clock::time_point t1 = std::chrono::steady_clock::now();
+    const Scalar timeDuration = std::chrono::duration<Scalar>(t1 - t0_).count();
+    if ((getMaximumTimeDuration() > 0.0) && (timeDuration > getMaximumTimeDuration()))
+      throw TimeoutException(HERE) << "Duration (" << timeDuration << "s) exceeds maximum duration (" << getMaximumTimeDuration() << " s)";
+
     if (stopCallback_.first && stopCallback_.first(stopCallback_.second))
-      throw InternalException(HERE) << "User stopped simulation";
+      throw InterruptionException(HERE) << "User stopped simulation";
   }
   ++ numberOfSteps_;
 
@@ -169,12 +153,6 @@ void SubsetSampling::run()
   // cannot determine next subset domain if no variance
   stop = stop || (std::abs(varianceEstimate) < epsilon);
 
-  if (iSubset_)
-  {
-    Scalar correction = ChiSquare(standardEvent_.getImplementation()->getAntecedent().getDistribution().getDimension()).computeComplementaryCDF(betaMin_ * betaMin_);
-    probabilityEstimate *= correction;
-  }
-
   // if there is no subset step ...
   if (stop)
   {
@@ -214,9 +192,7 @@ void SubsetSampling::run()
 
     // make sure the last failure domain does not overlap the real failure domain
     if (stop)
-    {
       currentThreshold = getEvent().getThreshold();
-    }
 
     // compute probability estimate on the current sample and group seeds at the beginning of the work sample
     previousVariance = varianceEstimate;
@@ -247,6 +223,20 @@ void SubsetSampling::run()
     if (probabilityEstimate < minimumProbability_)
       throw NotDefinedException(HERE) << "Probability estimate too small: " << probabilityEstimate;
 
+    std::chrono::steady_clock::time_point t1 = std::chrono::steady_clock::now();
+    const Scalar timeDuration = std::chrono::duration<Scalar>(t1 - t0_).count();
+    if ((getMaximumTimeDuration() > 0.0) && (timeDuration > getMaximumTimeDuration()))
+    {
+      LOGINFO("Simulation timeout");
+      stop = true;
+    }
+
+    if (stopCallback_.first && stopCallback_.first(stopCallback_.second))
+    {
+      LOGINFO("User stopped simulation");
+      stop = true;
+    }
+
     // compute variance estimate
     varianceEstimate = coefficientOfVariationSquare * pow(probabilityEstimate, 2.0);
 
@@ -263,6 +253,10 @@ void SubsetSampling::run()
 
   setResult(SubsetSamplingResult(getEvent(), probabilityEstimate, varianceEstimate, numberOfSteps_ * getMaximumOuterSampling(), getBlockSize(), sqrt(coefficientOfVariationSquare)));
 
+  std::chrono::steady_clock::time_point t1 = std::chrono::steady_clock::now();
+  const Scalar timeDuration = std::chrono::duration<Scalar>(t1 - t0_).count();
+  result_.setTimeDuration(timeDuration);
+
   // free work samples
   currentLevelSample_.clear();
   currentPointSample_.clear();
@@ -390,7 +384,7 @@ void SubsetSampling::generatePoints(Scalar threshold)
 {
   const UnsignedInteger maximumOuterSampling = getMaximumOuterSampling();
   const UnsignedInteger blockSize = getBlockSize();
-  const Distribution randomWalk(ComposedDistribution(ComposedDistribution::DistributionCollection(dimension_, Uniform(-0.5 * proposalRange_, 0.5 * proposalRange_))));
+  const Distribution randomWalk(JointDistribution(JointDistribution::DistributionCollection(dimension_, Uniform(-0.5 * proposalRange_, 0.5 * proposalRange_))));
   const UnsignedInteger Nc = seedNumber_;
 
   for (UnsignedInteger i = 0; i < maximumOuterSampling; ++ i)
@@ -418,9 +412,7 @@ void SubsetSampling::generatePoints(Scalar threshold)
 
         // accept new point with probability ratio
         if (ratio < uniform[k])
-        {
           newPoint[k] = oldPoint[k];
-        }
       }
 
       inputSample[j] = newPoint;
@@ -438,8 +430,13 @@ void SubsetSampling::generatePoints(Scalar threshold)
       }
     }
 
+    std::chrono::steady_clock::time_point t1 = std::chrono::steady_clock::now();
+    const Scalar timeDuration = std::chrono::duration<Scalar>(t1 - t0_).count();
+    if ((getMaximumTimeDuration() > 0.0) && (timeDuration > getMaximumTimeDuration()))
+      throw TimeoutException(HERE) << "Duration (" << timeDuration << "s) exceeds maximum duration (" << getMaximumTimeDuration() << " s)";
+
     if (stopCallback_.first && stopCallback_.first(stopCallback_.second))
-      throw InternalException(HERE) << "User stopped simulation";
+      throw InterruptionException(HERE) << "User stopped simulation";
   }
 }
 
@@ -515,27 +512,6 @@ Point SubsetSampling::getThresholdPerStep() const
 }
 
 
-void SubsetSampling::setKeepEventSample(const Bool keepEventSample)
-{
-  LOGWARN(OSS() << "SubsetSampling.setKeepEventSample is deprecated");
-  setKeepSample(keepEventSample);
-}
-
-
-Sample SubsetSampling::getEventInputSample() const
-{
-  LOGWARN(OSS() << "SubsetSampling.getEventInputSample is deprecated");
-  return getInputSample(getStepsNumber() - 1, 1);
-}
-
-
-Sample SubsetSampling::getEventOutputSample() const
-{
-  LOGWARN(OSS() << "SubsetSampling.getEventOutputSample is deprecated");
-  return getOutputSample(getStepsNumber() - 1, 1);
-}
-
-
 /* Keep event sample */
 void SubsetSampling::setKeepSample(const Bool keepSample)
 {
@@ -576,19 +552,6 @@ Indices SubsetSampling::getSampleIndices(const UnsignedInteger step, const Bool
   return result;
 }
 
-void SubsetSampling::setISubset(Bool iSubset)
-{
-  LOGWARN(OSS() << "SubsetSampling.setISubset is deprecated");
-  iSubset_ = iSubset;
-}
-
-void SubsetSampling::setBetaMin(Scalar betaMin)
-{
-  LOGWARN(OSS() << "SubsetSampling.setBetaMin is deprecated");
-  if (!(betaMin > 0.0)) throw InvalidArgumentException(HERE) << "Beta min should be positive";
-  betaMin_ = betaMin;
-}
-
 /* Experiment for first step */
 void SubsetSampling::setInitialExperiment(const WeightedExperiment & initialExperiment)
 {
@@ -623,8 +586,6 @@ void SubsetSampling::save(Advocate & adv) const
   EventSimulation::save(adv);
   adv.saveAttribute("proposalRange_", proposalRange_);
   adv.saveAttribute("conditionalProbability_", conditionalProbability_);
-  adv.saveAttribute("iSubset_", iSubset_);
-  adv.saveAttribute("betaMin_", betaMin_);
   adv.saveAttribute("minimumProbability_", minimumProbability_);
   adv.saveAttribute("initialExperiment_", initialExperiment_);
 
@@ -646,8 +607,6 @@ void SubsetSampling::load(Advocate & adv)
   EventSimulation::load(adv);
   adv.loadAttribute("proposalRange_", proposalRange_);
   adv.loadAttribute("conditionalProbability_", conditionalProbability_);
-  adv.loadAttribute("iSubset_", iSubset_);
-  adv.loadAttribute("betaMin_", betaMin_);
   adv.loadAttribute("minimumProbability_", minimumProbability_);
   if (adv.hasAttribute("initialExperiment_"))
     adv.loadAttribute("initialExperiment_", initialExperiment_);
