@@ -2,7 +2,7 @@
 /**
  *  @brief The ConditionalDistribution distribution
  *
- *  Copyright 2005-2023 Airbus-EDF-IMACS-ONERA-Phimeca
+ *  Copyright 2005-2024 Airbus-EDF-IMACS-ONERA-Phimeca
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -20,7 +20,7 @@
  */
 #include <cmath>
 #include "openturns/ConditionalDistribution.hxx"
-#include "openturns/ComposedDistribution.hxx"
+#include "openturns/JointDistribution.hxx"
 #include "openturns/Dirac.hxx"
 #include "openturns/Uniform.hxx"
 #include "openturns/Box.hxx"
@@ -198,7 +198,6 @@ void ConditionalDistribution::setConditionedAndConditioningDistributionsAndLinkF
   linkFunction_ = linkFunction;
   setDimension(conditioningDimension + conditionedDistribution.getDimension());
   // Start the discretisation into a Mixture
-  const Interval bounds(conditioningDistribution.getRange());
   // Here, implements some knowledge based selection of the integration method
   // For now, only basic Legendre
   // Gather the indices of the discrete marginals
@@ -237,7 +236,7 @@ void ConditionalDistribution::setConditionedAndConditioningDistributionsAndLinkF
   UnsignedInteger continuousAtomsNumber = 0;
   if (continuousDimension > 0)
   {
-    const ComposedDistribution measure(Collection< Distribution >(continuousDimension, Uniform()));
+    const JointDistribution measure(Collection< Distribution >(continuousDimension, Uniform()));
     // Create the DOE for continuous integration
     const String method(ResourceMap::GetAsString("ConditionalDistribution-ContinuousDiscretizationMethod"));
     const UnsignedInteger maximumIntegrationNumber = ResourceMap::GetAsUnsignedInteger( "ConditionalDistribution-MaximumIntegrationNodesNumber" );
